START TRANSACTION ;

-- =========================================
-- preemptive cleaning

delete from study_groups
    where id not in (
        select distinct GroupID
            from students_groups
        union distinct
        select distinct GroupID
            from disciplines_groups
    );

delete from specializations
    where id not in (
        select study_groups.SpecializationID from study_groups
    );

-- ==========================================
-- create & init `years` table
CREATE TABLE IF NOT EXISTS `years` (
    `Num` int(11) NOT NULL,
    PRIMARY KEY (`Num`)
) ENGINE=InnoDB  DEFAULT CHARSET=utf8;

insert into years (Num)
    select distinct semesters.Year from semesters;

alter table semesters
    add key `Year_2` (`Year`);

ALTER TABLE semesters
    ADD CONSTRAINT semesters_ibfk_1 FOREIGN KEY (`Year`) REFERENCES `years` (`Num`);

-- ==========================================
-- extract semester specific group info
-- add table `groups_years`

CREATE TABLE IF NOT EXISTS `groups_years` (
    `GroupID` int(11) NOT NULL,
    `Year` int(11) NOT NULL,
    `SpecializationID` int(11) NOT NULL,
    `Name` varchar(50) CHARACTER SET utf8 DEFAULT NULL,
    PRIMARY KEY (`Year`, `GroupID`),
    KEY `GroupID` (`GroupID`),
    KEY `Year` (`Year`),
    KEY `SpecializationID` (`SpecializationID`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

ALTER TABLE groups_years
    ADD CONSTRAINT groups_years_ibfk_1 FOREIGN KEY (`GroupID`) REFERENCES `study_groups`(`ID`),
    ADD CONSTRAINT groups_years_ibfk_2 FOREIGN KEY (`Year`) REFERENCES `years`(`Num`),
    ADD CONSTRAINT groups_years_ibfk_3 FOREIGN KEY (`SpecializationID`) REFERENCES `specializations`(`ID`);

insert into groups_years (GroupID, Year, SpecializationID, Name)
    select distinct study_groups.ID, semesters.Year, study_groups.SpecializationID, study_groups.Name
        from study_groups
        inner join students_groups on students_groups.GroupID = study_groups.ID
        inner join semesters on semesters.ID = students_groups.SemesterID;

-- ==========================================
-- save faculty info

alter table study_groups
    add column FacultyID int(11) NOT NULL;

update study_groups
    set FacultyID = (
        select specializations.FacultyID
            from specializations
            where specializations.id = study_groups.SpecializationID
    );

alter table study_groups
    add key FacultyID (`FacultyID`),
    add CONSTRAINT study_groups_ibfk_3 FOREIGN KEY (`FacultyID`) REFERENCES `faculties` (`ID`);

-- ==========================================
-- merge study_groups

delimiter //

create procedure mergeStudyGroups_mergeGroup(pNewGroupID int, pFacultyID int, pGradeID int, pGroupNum int) NO SQL
begin
    declare stOldGroupID int;

    declare done int default false;
    declare cur cursor for
        select ID
            from study_groups
            where
                ID <> pNewGroupID and
                FacultyID = pFacultyID and
                GradeID = pGradeID and
                GroupNum = pGroupNum;
    declare continue handler for not found set done = true;

    open cur;

    proc_loop: loop
        fetch cur into stOldGroupID;

        if done then
            leave proc_loop;
        end if;

        update logs_binds_groups
            set GroupID = pNewGroupID
            where GroupID = stOldGroupID;

        update students_groups
            set GroupID = pNewGroupID
            where GroupID = stOldGroupID;

        update ignore disciplines_groups
            set GroupID = pNewGroupID
            where GroupID = stOldGroupID;
        delete from disciplines_groups
            where GroupID = stOldGroupID;

        update groups_years
            set GroupID = pNewGroupID
            where GroupID = stOldGroupID;

        delete from study_groups
            where ID = stOldGroupID;
    end loop;

    close cur;
end//

create procedure mergeStudyGroups()
begin
    declare stGroupID, stFacultyID, stGradeID, stGroupNum int;

    declare done int default false;
    declare cur cursor for
        select min(ID), FacultyID, GradeID, GroupNum
            from study_groups
            group by FacultyID, GradeID, GroupNum
            having count(*) > 1;
    declare continue handler for not found set done = true;

    open cur;

    proc_loop: loop
        fetch cur into stGroupID, stFacultyID, stGradeID, stGroupNum;

        if done then
            leave proc_loop;
        end if;

        call mergeStudyGroups_mergeGroup(stGroupID, stFacultyID, stGradeID, stGroupNum);
    end loop;

    close cur;
end//

delimiter ;

call mergeStudyGroups();

drop procedure mergeStudyGroups_mergeGroup;
drop procedure mergeStudyGroups;

-- ==========================================
-- clear redundant columns in groups

alter table study_groups
    drop foreign key study_groups_ibfk_1,
    drop key SpecializationID,
    drop key GradeID_2;

alter table study_groups
    drop column SpecializationID,
    drop column Name;

alter table study_groups
    add unique key `FacultyGroup`(`FacultyID`,`GradeID`,`GroupNum`);

COMMIT ;
